module.exports = function Features(apiKey) {
  var axios  = require("axios")
  var stream = require("stream")
  var util   = require("util")

  var flags = {};

  function CallbackStream(opts) {
    this.opts = opts || {}

    if(!(this instanceof CallbackStream))
      return new CallbackStream(this.opts)

    stream.Writable.call(this, this.opts)
  }

  util.inherits(CallbackStream, stream.Writable)

  CallbackStream.prototype._write = function write(chunk, enc, next) {
    this.opts.cb(chunk.toString())

    next()
  }

  function send(cb) {
    const req = {
      method: "GET",
      headers: {
        "Authorization": apiKey,
        "accept": "text/event-stream"
      },
      url: "https://app.getcheckered.com/api/flags",
      responseType: "stream"
    };

    axios(req)
    .then(res => {
      res.data.pipe(CallbackStream({ cb: cb }));
      res.data.on("end", function() {
      })
    })
    .catch(error => console.log(error));
  }

  send(msg => {
    let { name, data } = JSON.parse(msg)

    switch(name) {
      case "toggle":
        // update individual flag
        flags[data.slug] = data
        break;
      default:
      case "init":
        // get list of flags
        for(var i = 0; i !== data.length; i++) {
          var flag = data[i]

          flags[flag.slug] = flag
        }
        break;
    }
  });

  function isEnabled(identifier) {
    return new Promise(function(resolve, reject) {
      resolve(flags[identifier] && flags[identifier].flag_on === true)
    })
  }

  return {
    isEnabled: isEnabled
  }
}
